from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


class Dashboard(models.Model):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255
    )
    ordering = models.PositiveIntegerField(
        verbose_name=_('ordering'),
        default=0
    )
    date_add = models.DateTimeField(
        verbose_name=_('date added'),
        default=timezone.now
    )

    class Meta:
        verbose_name = _('dashboard')
        verbose_name_plural = _('dashboards')
        ordering = ('ordering',)

    def __str__(self):
        return str(self.name)




ALL_FIELDS = '__all__'


EMPTY_VALUES = ([], (), {}, '', None)


class STRICTNESS(object):
    class IGNORE(object):
        pass

    class RETURN_NO_RESULTS(object):
        pass

    class RAISE_VALIDATION_ERROR(object):
        pass

    # Values of False & True chosen for backward compatability reasons.
    # Originally, these were the only options.
    _LEGACY = {
        False: IGNORE,
        True: RETURN_NO_RESULTS,
        "RAISE": RAISE_VALIDATION_ERROR,
    }



from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


class MenuSettings(models.Model):
    items = models.TextField(
        verbose_name=_('items'),
        blank=True,
        default=''
    )
    date_add = models.DateTimeField(
        verbose_name=_('date added'),
        default=timezone.now
    )

    class Meta:
        verbose_name = _('menu settings')
        verbose_name_plural = _('menu settings')

    def __str__(self):
        return '{}'.format(self.date_add)



from __future__ import absolute_import
from __future__ import unicode_literals

from collections import Iterable
from itertools import chain
try:
    from urllib.parse import urlencode
except:
    from urllib import urlencode  # noqa

import django
from django import forms
from django.db.models.fields import BLANK_CHOICE_DASH

from django.forms.utils import flatatt

from django.utils.datastructures import MultiValueDict
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.utils.six import string_types
from django.utils.translation import ugettext as _

from .compat import format_value


class LinkWidget(forms.Widget):
    def __init__(self, attrs=None, choices=()):
        super(LinkWidget, self).__init__(attrs)

        self.choices = choices

    def value_from_datadict(self, data, files, name):
        value = super(LinkWidget, self).value_from_datadict(data, files, name)
        self.data = data
        return value

    def render(self, name, value, attrs=None, choices=()):
        if not hasattr(self, 'data'):
            self.data = {}
        if value is None:
            value = ''
        if django.VERSION < (1, 11):
            final_attrs = self.build_attrs(attrs)
        else:
            final_attrs = self.build_attrs(self.attrs, extra_attrs=attrs)
        output = ['<ul%s>' % flatatt(final_attrs)]
        options = self.render_options(choices, [value], name)
        if options:
            output.append(options)
        output.append('</ul>')
        return mark_safe('\n'.join(output))

    def render_options(self, choices, selected_choices, name):
        selected_choices = set(force_text(v) for v in selected_choices)
        output = []
        for option_value, option_label in chain(self.choices, choices):
            if isinstance(option_label, (list, tuple)):
                for option in option_label:
                    output.append(
                        self.render_option(name, selected_choices, *option))
            else:
                output.append(
                    self.render_option(name, selected_choices,
                                       option_value, option_label))
        return '\n'.join(output)

    def render_option(self, name, selected_choices,
                      option_value, option_label):
        option_value = force_text(option_value)
        if option_label == BLANK_CHOICE_DASH[0][1]:
            option_label = _("All")
        data = self.data.copy()
        data[name] = option_value
        selected = data == self.data or option_value in selected_choices
        try:
            url = data.urlencode()
        except AttributeError:
            url = urlencode(data)
        return self.option_string() % {
            'attrs': selected and ' class="selected"' or '',
            'query_string': url,
            'label': force_text(option_label)
        }

    def option_string(self):
        return '<li><a%(attrs)s href="?%(query_string)s">%(label)s</a></li>'


class RangeWidget(forms.MultiWidget):
    template_name = 'jet_django.deps.django_filters/widgets/multiwidget.html'

    def __init__(self, attrs=None):
        widgets = (forms.TextInput, forms.TextInput)
        super(RangeWidget, self).__init__(widgets, attrs)

    def format_output(self, rendered_widgets):
        # Method was removed in Django 1.11.
        return '-'.join(rendered_widgets)

    def decompress(self, value):
        if value:
            return [value.start, value.stop]
        return [None, None]


class LookupTypeWidget(forms.MultiWidget):
    def decompress(self, value):
        if value is None:
            return [None, None]
        return value


class BooleanWidget(forms.Select):
    """Convert true/false values into the internal Python True/False.
    This can be used for AJAX queries that pass true/false from JavaScript's
    internal types through.
    """
    def __init__(self, attrs=None):
        choices = (('', _('Unknown')),
                   ('true', _('Yes')),
                   ('false', _('No')))
        super(BooleanWidget, self).__init__(attrs, choices)

    def render(self, name, value, attrs=None):
        try:
            value = {
                True: 'true',
                False: 'false',
                '1': 'true',
                '0': 'false'
            }[value]
        except KeyError:
            value = ''
        return super(BooleanWidget, self).render(name, value, attrs)

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        if isinstance(value, string_types):
            value = value.lower()

        return {
            '1': True,
            '0': False,
            'true': True,
            'false': False,
            True: True,
            False: False,
        }.get(value, None)


class BaseCSVWidget(forms.Widget):
    def _isiterable(self, value):
        return isinstance(value, Iterable) and not isinstance(value, string_types)

    def value_from_datadict(self, data, files, name):
        value = super(BaseCSVWidget, self).value_from_datadict(data, files, name)

        if value is not None:
            if value == '':  # empty value should parse as an empty list
                return []
            return value.split(',')
        return None

    def render(self, name, value, attrs=None):
        if not self._isiterable(value):
            value = [value]

        if len(value) <= 1:
            # delegate to main widget (Select, etc...) if not multiple values
            value = value[0] if value else ''
            return super(BaseCSVWidget, self).render(name, value, attrs)

        # if we have multiple values, we need to force render as a text input
        # (otherwise, the additional values are lost)
        surrogate = forms.TextInput()
        value = [force_text(format_value(surrogate, v)) for v in value]
        value = ','.join(list(value))

        return surrogate.render(name, value, attrs)


class CSVWidget(BaseCSVWidget, forms.TextInput):
    pass


class QueryArrayWidget(BaseCSVWidget, forms.TextInput):
    """
    Enables request query array notation that might be consumed by MultipleChoiceFilter

    1. Values can be provided as csv string:  ?foo=bar,baz
    2. Values can be provided as query array: ?foo[]=bar&foo[]=baz
    3. Values can be provided as query array: ?foo=bar&foo=baz

    Note: Duplicate and empty values are skipped from results
    """

    def value_from_datadict(self, data, files, name):
        if not isinstance(data, MultiValueDict):
            for key, value in data.items():
                # treat value as csv string: ?foo=1,2
                if isinstance(value, string_types):
                    data[key] = [x.strip() for x in value.rstrip(',').split(',') if x]
            data = MultiValueDict(data)

        values_list = data.getlist(name, data.getlist('%s[]' % name)) or []

        # apparently its an array, so no need to process it's values as csv
        # ?foo=1&foo=2 -> data.getlist(foo) -> foo = [1, 2]
        # ?foo[]=1&foo[]=2 -> data.getlist(foo[]) -> foo = [1, 2]
        if len(values_list) > 0:
            ret = [x for x in values_list if x]
        else:
            ret = []

        return list(set(ret))



from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


class Widget(models.Model):
    dashboard = models.ForeignKey(
        to='jet_django.Dashboard',
        verbose_name=_('dashboard'),
        related_name='widgets'
    )
    widget_type = models.CharField(
        verbose_name=_('type'),
        max_length=255
    )
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255
    )
    x = models.PositiveSmallIntegerField(
        verbose_name=_('x')
    )
    y = models.PositiveSmallIntegerField(
        verbose_name=_('y')
    )
    width = models.PositiveSmallIntegerField(
        verbose_name=_('width'),
        default=1
    )
    height = models.PositiveSmallIntegerField(
        verbose_name=_('height'),
        default=1
    )
    params = models.TextField(
        verbose_name=_('params'),
        blank=True,
        default=''
    )
    date_add = models.DateTimeField(
        verbose_name=_('date added'),
        default=timezone.now
    )

    class Meta:
        verbose_name = _('widget')
        verbose_name_plural = _('widgets')
        ordering = ('y', 'x')

    def __str__(self):
        return str(self.name)



from django.conf.urls import include, url

from jet_django.deps.rest_framework.renderers import (
    CoreJSONRenderer, DocumentationRenderer, SchemaJSRenderer
)
from jet_django.deps.rest_framework.schemas import get_schema_view


def get_docs_view(title=None, description=None, schema_url=None, public=True):
    renderer_classes = [DocumentationRenderer, CoreJSONRenderer]

    return get_schema_view(
        title=title,
        url=schema_url,
        description=description,
        renderer_classes=renderer_classes,
        public=public
    )


def get_schemajs_view(title=None, description=None, schema_url=None, public=True):
    renderer_classes = [SchemaJSRenderer]

    return get_schema_view(
        title=title,
        url=schema_url,
        description=description,
        renderer_classes=renderer_classes,
        public=public
    )


def include_docs_urls(title=None, description=None, schema_url=None, public=True):
    docs_view = get_docs_view(
        title=title,
        description=description,
        schema_url=schema_url,
        public=public
    )
    schema_js_view = get_schemajs_view(
        title=title,
        description=description,
        schema_url=schema_url,
        public=public
    )
    urls = [
        url(r'^$', docs_view, name='docs-index'),
        url(r'^schema.js$', schema_js_view, name='schema-js')
    ]
    return include(urls, namespace='api-docs')



from django.http import Http404
from django.shortcuts import get_object_or_404
from jet_django.deps.rest_framework import viewsets

from jet_django.models.menu_item import MenuSettings
from jet_django.permissions import HasProjectPermissions
from jet_django.serializers.menu_settings import MenuSettingsSerializer


class MenuSettingsViewSet(viewsets.ModelViewSet):
    model = MenuSettings
    serializer_class = MenuSettingsSerializer
    queryset = MenuSettings.objects.all()
    pagination_class = None
    authentication_classes = ()
    permission_classes = (HasProjectPermissions,)

    def create(self, request, *args, **kwargs):
        try:
            self.get_object()
            return super().update(request, *args, **kwargs)
        except Http404:
            return super().create(request, *args, **kwargs)

    def get_object(self):
        queryset = self.get_queryset()[:1]
        obj = get_object_or_404(queryset)

        # May raise a permission denied
        self.check_object_permissions(self.request, obj)

        return obj



from django.http import Http404
from django.shortcuts import get_object_or_404
from jet_django.deps.rest_framework import viewsets

from jet_django.filters.view_settings import ViewSettingsFilterSet
from jet_django.models.view_settings import ViewSettings
from jet_django.permissions import HasProjectPermissions
from jet_django.serializers.view_settings import ViewSettingsSerializer


class ViewSettingsViewSet(viewsets.ModelViewSet):
    model = ViewSettings
    serializer_class = ViewSettingsSerializer
    queryset = ViewSettings.objects
    filter_class = ViewSettingsFilterSet
    pagination_class = None
    authentication_classes = ()
    permission_classes = (HasProjectPermissions,)

    def create(self, request, *args, **kwargs):
        try:
            object = self.get_object()
            return super().update(request, *args, **kwargs)
        except Http404:
            return super().create(request, *args, **kwargs)

    def get_object(self):
        queryset = self.filter_queryset(self.get_queryset())
        filter_kwargs = {
            'app_label': self.request.data.get('app_label'),
            'model': self.request.data.get('model'),
            'view': self.request.data.get('view')
        }
        obj = get_object_or_404(queryset, **filter_kwargs)

        # May raise a permission denied
        self.check_object_permissions(self.request, obj)

        return obj



from jet_django.deps.rest_framework import serializers


class RawFileField(serializers.CharField):

    def to_representation(self, value):
        return serializers.FileField.to_representation(self, value)



from jet_django.deps.rest_framework import serializers

from jet_django.models.view_settings import ViewSettings


class ViewSettingsSerializer(serializers.ModelSerializer):
    class Meta:
        model = ViewSettings
        fields = (
            'app_label',
            'model',
            'view',
            'params',
            'date_add'
        )



from django.db import connection, ProgrammingError
from django.utils.translation import ugettext_lazy as _
from django.utils import six
from jet_django.deps.rest_framework import views, status
from jet_django.deps.rest_framework.exceptions import APIException
from jet_django.deps.rest_framework.response import Response

from jet_django.permissions import HasProjectPermissions
from jet_django.serializers.sql import SqlSerializer


class SqlError(APIException):
    status_code = status.HTTP_400_BAD_REQUEST
    default_detail = _('Query failed')
    default_code = 'invalid'

    def __init__(self, detail):
        self.detail = {'error': str(detail)}

    def __str__(self):
        return six.text_type(self.detail)


class SqlView(views.APIView):
    pagination_class = None
    authentication_classes = ()
    permission_classes = (HasProjectPermissions,)

    def post(self, request, *args, **kwargs):
        serializer = SqlSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        with connection.cursor() as cursor:
            try:
                cursor.execute(serializer.data['query'], serializer.data.get('params', '').split(','))
            except ProgrammingError as e:
                raise SqlError(e)
            rows = cursor.fetchall()

            return Response(rows)



from __future__ import absolute_import
from __future__ import unicode_literals

from datetime import datetime, time
from collections import namedtuple

from django import forms
from django.utils.dateparse import parse_datetime

from django.utils.encoding import force_str
from django.utils.translation import ugettext_lazy as _

from .utils import handle_timezone
from .widgets import RangeWidget, LookupTypeWidget, CSVWidget, BaseCSVWidget


class RangeField(forms.MultiValueField):
    widget = RangeWidget

    def __init__(self, fields=None, *args, **kwargs):
        if fields is None:
            fields = (
                forms.DecimalField(),
                forms.DecimalField())
        super(RangeField, self).__init__(fields, *args, **kwargs)

    def compress(self, data_list):
        if data_list:
            return slice(*data_list)
        return None


class DateRangeField(RangeField):

    def __init__(self, *args, **kwargs):
        fields = (
            forms.DateField(),
            forms.DateField())
        super(DateRangeField, self).__init__(fields, *args, **kwargs)

    def compress(self, data_list):
        if data_list:
            start_date, stop_date = data_list
            if start_date:
                start_date = handle_timezone(
                    datetime.combine(start_date, time.min),
                    False
                )
            if stop_date:
                stop_date = handle_timezone(
                    datetime.combine(stop_date, time.max),
                    False
                )
            return slice(start_date, stop_date)
        return None


class DateTimeRangeField(RangeField):

    def __init__(self, *args, **kwargs):
        fields = (
            forms.DateTimeField(),
            forms.DateTimeField())
        super(DateTimeRangeField, self).__init__(fields, *args, **kwargs)


class TimeRangeField(RangeField):

    def __init__(self, *args, **kwargs):
        fields = (
            forms.TimeField(),
            forms.TimeField())
        super(TimeRangeField, self).__init__(fields, *args, **kwargs)


class Lookup(namedtuple('Lookup', ('value', 'lookup_type'))):
    # python nature is test __len__ on tuple types for boolean check
    def __len__(self):
        if not self.value:
            return 0
        return 2


class LookupTypeField(forms.MultiValueField):
    def __init__(self, field, lookup_choices, *args, **kwargs):
        fields = (
            field,
            forms.ChoiceField(choices=lookup_choices)
        )
        defaults = {
            'widgets': [f.widget for f in fields],
        }
        widget = LookupTypeWidget(**defaults)
        kwargs['widget'] = widget
        kwargs['help_text'] = field.help_text
        super(LookupTypeField, self).__init__(fields, *args, **kwargs)

    def compress(self, data_list):
        if len(data_list) == 2:
            return Lookup(value=data_list[0], lookup_type=data_list[1] or 'exact')
        return Lookup(value=None, lookup_type='exact')


class IsoDateTimeField(forms.DateTimeField):
    """
    Supports 'iso-8601' date format too which is out the scope of
    the ``datetime.strptime`` standard library

    # ISO 8601: ``http://www.w3.org/TR/NOTE-datetime``

    Based on Gist example by David Medina https://gist.github.com/copitux/5773821
    """
    ISO_8601 = 'iso-8601'
    input_formats = [ISO_8601]

    def strptime(self, value, format):
        value = force_str(value)

        if format == self.ISO_8601:
            parsed = parse_datetime(value)
            if parsed is None:  # Continue with other formats if doesn't match
                raise ValueError
            return handle_timezone(parsed)
        return super(IsoDateTimeField, self).strptime(value, format)


class BaseCSVField(forms.Field):
    """
    Base field for validating CSV types. Value validation is performed by
    secondary base classes.

    ex::
        class IntegerCSVField(BaseCSVField, filters.IntegerField):
            pass

    """
    base_widget_class = BaseCSVWidget

    def __init__(self, *args, **kwargs):
        widget = kwargs.get('widget') or self.widget
        kwargs['widget'] = self._get_widget_class(widget)

        super(BaseCSVField, self).__init__(*args, **kwargs)

    def _get_widget_class(self, widget):
        # passthrough, allows for override
        if isinstance(widget, BaseCSVWidget) or (
                isinstance(widget, type) and
                issubclass(widget, BaseCSVWidget)):
            return widget

        # complain since we are unable to reconstruct widget instances
        assert isinstance(widget, type), \
            "'%s.widget' must be a widget class, not %s." \
            % (self.__class__.__name__, repr(widget))

        bases = (self.base_widget_class, widget, )
        return type(str('CSV%s' % widget.__name__), bases, {})

    def clean(self, value):
        if value is None:
            return None
        return [super(BaseCSVField, self).clean(v) for v in value]


class BaseRangeField(BaseCSVField):
    # Force use of text input, as range must always have two inputs. A date
    # input would only allow a user to input one value and would always fail.
    widget = CSVWidget

    default_error_messages = {
        'invalid_values': _('Range query expects two values.')
    }

    def clean(self, value):
        value = super(BaseRangeField, self).clean(value)

        if value is not None and len(value) != 2:
            raise forms.ValidationError(
                self.error_messages['invalid_values'],
                code='invalid_values')

        return value



from jet_django.deps.rest_framework import views
from jet_django.deps.rest_framework.response import Response

from jet_django.permissions import HasProjectPermissions
from jet_django.serializers.file_upload import FileUploadSerializer


class FileUploadView(views.APIView):
    pagination_class = None
    authentication_classes = ()
    permission_classes = (HasProjectPermissions,)

    def post(self, request, *args, **kwargs):
        serializer = FileUploadSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data)
